# NOTE: Derived from blib/lib/Business/WorldPay/Junior.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package Business::WorldPay::Junior;

#line 245 "blib/lib/Business/WorldPay/Junior.pm (autosplit into blib/lib/auto/Business/WorldPay/Junior/callback.al)"
sub callback
    {
    # verifies the validity of data supplied via the callback facility.

    my $self = shift;
    my $hr_callback = shift;	# Hash of the values returned by callback
    
    # Are all the mandatory callback fields present and all data valid?

    my %required = ( instId => 1,
                     cartId => 1,
                     desc => 1,
                     transStatus => 1,
                   );
    
    my %rules = ( instId => '\d+',
                  cartId => '\d+',
                  currency => '(GBP|EUR|USD)',
                  amount => '[\d\.]+',
                  desc => '[[:print:]]+',
                  testMode => '(0|100|101)',
                  name => '[[:print:]]{0,40}',
                  address => '[[:print:]]{0,255}',
                  postcode => '[[:print:]]{0,12}',
                  country => '[A-Z]{2}',
                  tel => '[[:print:]]{0,30}',
                  fax => '[[:print:]]{0,30}',
                  email => '[[:print:]]{0,80}',
                  transId => '\d{1,16}',
                  futurePayId => '\d{1,16}',
                  transStatus => '(Y|C)',
                  transTime => '\d+',
                  authAmount => '[\d\.]+',
                  authCurrency => '(GBP|EUR|USD)',
                  cardType => '[[:print:]]+',
                  AVS => '\d{4}',
                );
                  
    my @fields = ( keys %rules );

    if ( $hr_callback->{transStatus} eq 'Y' )
        {
        $required{authCurrency} = 1;
        $required{authAmount} = 1;
        $required{transTime} = 1;
        $required{transId} = 1;
        }
    
    foreach my $field (@fields)
        {
        if ( defined $required{$field} && ! $hr_callback->{$field} )
            {
            $errstr = "mandatory field $field is missing";
            return;
            }
        next if ! $hr_callback->{$field};

        if ( $hr_callback->{$field} !~ m/^$rules{$field}$/i )
            {
            $errstr = "field $field invalid";
            return;
            }
        }
    
    # retrieve the stored details for the transaction
    my $rdb = $self->db_connect;
    my $sql = sprintf "SELECT instId, currency, amount, description, testMode FROM worldpay WHERE cartId = %s", $rdb->quote($hr_callback->{cartId});
    my $sth = $rdb->prepare($sql);
    if ( ! $sth )
        {
        $errstr = "not able to prepare $sql";
        $sth->finish;
        $rdb->disconnect if $rdb;;
        return;
        }
    if ( ! $sth->execute )
        {
        $errstr = "not able to execute prepared $sql";
        $sth->finish;
        $rdb->disconnect if $rdb;;
        return;
        }
        
    my $hr_trans = $sth->fetchrow_hashref();
    if ( ! $hr_trans )
        {
        $errstr = "no data in database - $sql -\n";
        $sth->finish;
        $rdb->disconnect if $rdb;;
        return;
        }
    
    # Is the payment correct?
    my $fail = undef;	# flag to indicate that callback does not match
			# stored transaction details
			
    $fail = 1 if $hr_trans->{instId} != $hr_callback->{instId};
    $fail = 1 if $hr_trans->{testMode} != $hr_callback->{testMode};
    $fail = 1 if $hr_trans->{description} ne $hr_callback->{desc};

    # If the transaction was cancelled that's all we'll have so we need
    # to test for that now and act accordingly.

    if ( $hr_callback->{transStatus} ne 'Y' )
        {
        # The transaction was cancelled.
        # Update the db to indicate this and return false
        $sql = sprintf "UPDATE worldpay SET transStatus = 'C' WHERE cartId = %s", $rdb->quote($hr_callback->{cartId});
        eval ( $rdb->do($sql) );
        if ( $@ )
            {
            $errstr = "transaction failed - unable to update DB!";
            $rdb->disconnect if $rdb;;
            return;
            }
        $rdb->disconnect if $rdb;;
        return 1;	# Database updated to reflect transaction cancelled.
        		# call $self->authorised() for a true/false on auth
        }

    $fail = 1 if $hr_trans->{amount} != $hr_callback->{amount};
    $fail = 1 if $hr_trans->{currency} ne $hr_callback->{currency};
    
    if ( $fail )
        {
        $errstr = "db and callback mismatch";
        $sth->finish;
        $rdb->disconnect if $rdb;;
        return;
        }

    $sth->finish;
    
    # At this point we know that the transaction does match the callback.
    # We also know that the desc and instId fields are correct and that the
    # transaction was authorised
    
    # Before we test anything else we need to double check the transaction.
    
    if ( $hr_callback->{transStatus} eq 'Y' )
        {
        # We need to check the amount and currency fields to verify the transaction
        
        if ( $hr_trans->{currency} ne $hr_callback->{authCurrency} )
            {
            # check to see whether the transaction is in another currency
            # and if so whether the amount is correct after exchange rate
            # calculations.
            if ( ! &exchange_rate( $hr_trans, $hr_callback ) )
                {
                $errstr = "Invalid transaction. Amount paid either insufficient or no exchange rate date available";
                return undef;
		}
            }
        
        # OK the payment has been accepted and is sufficient. We're happy.
        # Record the payment details and return true.
        
        # Take the values from the callback that we care about and store them
        # in the database.

        my $count = 1;
        my $sql = "UPDATE worldpay SET";
        foreach (@fields)
            {
            next if $_ =~ /(desc|cartId|amount|currency|instId)/;
            next if ! defined $hr_callback->{$_};
            $sql .= "," if $count > 1;
            $sql .= " $_ = " . $rdb->quote($hr_callback->{$_});
            $count++;
            }
        $sql .= " WHERE cartId = " . $rdb->quote($hr_callback->{cartId});
        eval ( $rdb->do($sql) );
        if ( $@ )
            {
            $errstr = "transaction authorised - unable to update DB!";
            $rdb->disconnect if $rdb;;
            return;
            }
        $rdb->disconnect if $rdb;;
        return 1;
        }
    }

# end of Business::WorldPay::Junior::callback
1;
